<?php

declare(strict_types=1);

namespace BitBag\SyliusBonusPointsPlugin\Calculator;

use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategy;
use Sylius\Component\Core\Model\OrderInterface;
use Webmozart\Assert\Assert;

final class OrderTotalPercentageCalculator implements BonusPointsStrategyCalculatorInterface
{
    public function calculate($subject, array $configuration, int $amountToDeduct = 0): int
    {
        /** @var OrderInterface $subject */
        Assert::isInstanceOf($subject, OrderInterface::class);

        $configuration = $configuration[$subject->getChannel()->getCode()];

        $total = $subject->getTotal();

        if ($amountToDeduct < 0) {
            $total += $amountToDeduct;
        }

        $result = (int) floor($total * $configuration['percentToCalculatePoints']);

        return ($result === 0) ? 0 : (int) floor(($total * $configuration['percentToCalculatePoints']) / 100);
    }

    public function isPerOrderItem(): bool
    {
        return false;
    }

    public function getType(): string
    {
        return 'order_total_percentage';
    }

    public function getCompatibleStrategyTypes(): array
    {
        return [
            BonusPointsStrategy::TYPE_ORDER,
        ];
    }
}
